@@ -29,8 +29,19 @@ static unsigned long	netmask;
 /* Used by nfs.c */
 char *hostname = "";
 int hostnamelen = 0;
+#ifndef USE_STATIC_BOOT_INFO
 static uint32_t xid;
+#endif
+static uint16_t sport;
 unsigned char *end_of_rfc1533 = NULL;
+unsigned char *addparam;
+int addparamlen;
+
+#ifdef IMAGE_FREEBSD
+int freebsd_howto = 0;
+char freebsd_kernel_env[FREEBSD_KERNEL_ENV_SIZE];
+#endif /* IMAGE_FREEBSD */
+
 static int vendorext_isvalid;
 static const unsigned char vendorext_magic[] = {0xE4,0x45,0x74,0x68}; /* Eth */
 static const unsigned char broadcast[] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
@@ -38,14 +49,26 @@ static const in_addr zeroIP = { 0L };
 
 struct bootpd_t bootp_data;
 
+#ifdef PXE_DHCP_STRICT
+#define MAX_BOOT_MENU 5
+#define MAX_BOOT_ENTRY_LENGTH 60
+struct {
+  uint16_t id;
+  unsigned char text[MAX_BOOT_ENTRY_LENGTH+1];
+  in_addr ip;
+} pxe_boot_menu[MAX_BOOT_MENU];
+static unsigned char num_boot_menu = 0;
+#endif /* PXE_DHCP_STRICT */
+
 #ifdef	NO_DHCP_SUPPORT
 static unsigned char	rfc1533_cookie[5] = { RFC1533_COOKIE, RFC1533_END };
-#else	/* !NO_DHCP_SUPPORT */
+#else	/* NO_DHCP_SUPPORT */
 static int dhcp_reply;
 static in_addr dhcp_server = { 0L };
-static in_addr dhcp_addr = { 0L };
 static unsigned char	rfc1533_cookie[] = { RFC1533_COOKIE };
 #define DHCP_MACHINE_INFO_SIZE (sizeof dhcp_machine_info)
+#ifndef USE_STATIC_BOOT_INFO
+static in_addr dhcp_addr = { 0L };
 static unsigned char dhcp_machine_info[] = {
 	/* Our enclosing DHCP tag */
 	RFC1533_VENDOR_ETHERBOOT_ENCAP, 11,
@@ -60,6 +83,7 @@ static unsigned char dhcp_machine_info[] = {
 #define DHCP_MACHINE_INFO_SIZE (sizeof(dhcp_machine_info) - (EM_CURRENT_64_PRESENT? 0: 4))
 #endif /* EM_CURRENT_64 */
 };
+#endif /* USE_STATIC_BOOT_INFO */
 static const unsigned char dhcpdiscover[] = {
 	RFC2132_MSG_TYPE,1,DHCPDISCOVER,
 	RFC2132_MAX_SIZE,2,	/* request as much as we can */
@@ -216,6 +240,17 @@ static const unsigned char proxydhcprequest [] = {
 };
 #endif
 
+#ifdef PXE_DHCP_STRICT
+static const unsigned char pxedhcprequest [] = {
+	RFC2132_MSG_TYPE,1,DHCPREQUEST,
+	RFC1533_VENDOR,6,PXE_BOOT_ITEM,4,0,0,0,0,
+	RFC3679_PXE_CLIENT_UUID,RFC3679_PXE_CLIENT_UUID_LENGTH,RFC3679_PXE_CLIENT_UUID_DEFAULT,
+	RFC3679_PXE_CLIENT_ARCH,RFC3679_PXE_CLIENT_ARCH_LENGTH,RFC3679_PXE_CLIENT_ARCH_IAX86PC,
+	RFC3679_PXE_CLIENT_NDI, RFC3679_PXE_CLIENT_NDI_LENGTH, RFC3679_PXE_CLIENT_NDI_21,
+	RFC2132_VENDOR_CLASS_ID,RFC2132_VENDOR_CLASS_ID_PXE_LENGTH,RFC2132_VENDOR_CLASS_ID_PXE
+};
+#endif /* PXE_DHCP_STRICT */
+
 #ifdef	REQUIRE_VCI_ETHERBOOT
 int	vci_etherboot;
 #endif
@@ -270,8 +305,12 @@ struct nic	nic =
 #ifdef RARP_NOT_BOOTP
 static int rarp(void);
 #else
+#ifdef USE_STATIC_BOOT_INFO
+static int get_static_boot_info(void);
+#else
 static int bootp(void);
-#endif
+#endif /* USE_STATIC_BOOT_INFO */
+#endif /* RARP_NOT_BOOTP */
 static unsigned short tcpudpchksum(struct iphdr *ip);
 
 
@@ -314,6 +353,9 @@ int eth_load_configuration(struct dev *dev __unused)
 {
 	int server_found;
 	/* Find a server to get BOOTP reply from */
+#ifdef  USE_STATIC_BOOT_INFO
+	printf("Using Static IP Information...");
+#else	
 #ifdef	RARP_NOT_BOOTP
 	printf("Searching for server (RARP)...");
 #else
@@ -323,11 +365,16 @@ int eth_load_configuration(struct dev *dev __unused)
 	printf("Searching for server (BOOTP)...");
 #endif
 #endif
-
+#endif
+	
+#ifdef  USE_STATIC_BOOT_INFO
+	server_found = get_static_boot_info();
+#else	
 #ifdef	RARP_NOT_BOOTP
 	server_found = rarp();
 #else
 	server_found = bootp();
+#endif
 #endif
 	if (!server_found) {
 		printf("No Server found\n");
@@ -344,6 +391,7 @@ int eth_load(struct dev *dev __unused)
 {
 	const char	*kernel;
 	printf("\nMe: %@", arptable[ARP_CLIENT].ipaddr.s_addr );
+#ifndef USE_STATIC_BOOT_INFO
 #ifndef NO_DHCP_SUPPORT
 	printf(", DHCP: %@", dhcp_server );
 #ifdef PXE_EXPORT       
@@ -352,10 +400,10 @@ int eth_load(struct dev *dev __unused)
 		       arptable[ARP_PROXYDHCP].ipaddr.s_addr);
 #endif /* PXE_EXPORT */
 #endif /* ! NO_DHCP_SUPPORT */
+#endif /* USE_STATIC_BOOT_INFO */	
 	printf(", TFTP: %@", arptable[ARP_SERVER].ipaddr.s_addr);
-	if (BOOTP_DATA_ADDR->bootp_reply.bp_giaddr.s_addr)
-		printf(", Relay: %@",
-			BOOTP_DATA_ADDR->bootp_reply.bp_giaddr.s_addr);
+	if (bootp_data.bootp_reply.bp_giaddr.s_addr)
+		printf(", Relay: %@", bootp_data.bootp_reply.bp_giaddr.s_addr);
 	if (arptable[ARP_GATEWAY].ipaddr.s_addr)
 		printf(", Gateway %@", arptable[ARP_GATEWAY].ipaddr.s_addr);
 #ifdef	DNS_RESOLVER
@@ -372,13 +420,23 @@ int eth_load(struct dev *dev __unused)
 #ifdef	DOWNLOAD_PROTO_NFS
 	rpc_init();
 #endif
-	kernel = KERNEL_BUF[0] == '\0' ? 
-#ifdef	DEFAULT_BOOTFILE
-		DEFAULT_BOOTFILE
+
+	/* Handle boot filename static and default cases */
+
+	if ( KERNEL_BUF[0] != '\0' ) {
+	  kernel = KERNEL_BUF;
+	} else {
+#if defined(USE_STATIC_BOOT_INFO) && defined(STATIC_BOOTFILE)
+	  kernel = STATIC_BOOTFILE;
+#elif defined(DEFAULT_BOOTFILE)
+	  kernel = DEFAULT_BOOTFILE;
 #else
-		NULL
+	  kernel = NULL;
 #endif
-		: KERNEL_BUF;
+	}
+
+	/* Remove .zpxe from filename if it exists */
+
 #ifdef  ZPXE_SUFFIX_STRIP
         {
           int i = 0;
@@ -395,9 +453,13 @@ int eth_load(struct dev *dev __unused)
           }
         }
 #endif
+
 	if ( kernel ) {
 		loadkernel(kernel); /* We don't return except on error */
 		printf("Unable to load file.\n");
+#ifdef EXIT_ON_FILE_LOAD_ERROR
+		longjmp(restart_etherboot, 255);
+#endif
 	} else {	
 		printf("No filename\n");
 	}
@@ -572,6 +634,7 @@ UDP_TRANSMIT - Send an UDP datagram
 int udp_transmit(unsigned long destip, unsigned int srcsock,
 	unsigned int destsock, int len, const void *buf)
 {
+	sport = srcsock;
 	build_udp_hdr(destip, srcsock, destsock, 60, len, buf);
 	return ip_transmit(len, buf);
 }
@@ -788,7 +851,33 @@ int tftp_block ( struct tftpreq_info_t *request, struct tftpblk_info_t *block )
 }
 #endif	/* DOWNLOAD_PROTO_TFTP */
 
+#ifdef  USE_STATIC_BOOT_INFO
+/***************************************************************************
+get_static_boot_info - construct bootp response from supplied CFLAG values
+***************************************************************************/
+static int get_static_boot_info (void)
+{
+	in_addr client_addr, server_addr, gateway_addr, netmask_addr;
+
+	inet_aton( STATIC_CLIENT_IP, &client_addr );
+	memcpy ( &arptable[ARP_CLIENT].ipaddr, &client_addr.s_addr, sizeof(in_addr) );
+
+	inet_aton( STATIC_SERVER_IP, &server_addr );
+	memcpy( &arptable[ARP_SERVER].ipaddr, &server_addr.s_addr, sizeof(in_addr) );
+
+	inet_aton( STATIC_GATEWAY_IP, &gateway_addr );
+	memcpy( &arptable[ARP_GATEWAY].ipaddr, &gateway_addr.s_addr, sizeof(in_addr) );
+
+	inet_aton( STATIC_SUBNET_MASK, &netmask_addr );
+	netmask = ntohl( netmask_addr.s_addr );
+
+	bootp_data.bootp_reply.bp_op = BOOTP_REPLY;
+
+	return (1);
+}
+#endif
 #ifdef	RARP_NOT_BOOTP
+
 /**************************************************************************
 RARP - Get my IP address and load information
 **************************************************************************/
@@ -852,6 +941,22 @@ static int rarp(void)
 
 #else
 
+#ifdef PXE_DHCP_STRICT
+/**************************************************************************
+FIND_VCI_PXECLIENT - Looks for "PXEClient" Vendor Class Identifier
+Upon entry p points after rfc1533_cookie.
+**************************************************************************/
+static int find_vci_pxeclient(unsigned char*p)
+{
+	for (; *p != RFC1533_END; p += TAG_LEN(p) + 2)
+		if (*p == RFC2132_VENDOR_CLASS_ID)
+			if (memcmp(p + 2, "PXEClient", 9) == 0)
+				return 1;
+	return 0;
+}
+#endif
+
+#ifndef USE_STATIC_BOOT_INFO
 /**************************************************************************
 BOOTP - Get my IP address and load information
 **************************************************************************/
@@ -875,12 +980,30 @@ static int await_bootp(int ival __unused, void *ptr __unused,
 		) {
 		return 0;
 	}
-	if (udp->dest != htons(BOOTP_CLIENT))
+	if (udp->dest != htons(sport))
 		return 0;
 	if (bootpreply->bp_op != BOOTP_REPLY)
 		return 0;
 	if (bootpreply->bp_xid != xid)
 		return 0;
+#ifdef PXE_DHCP_STRICT
+	/* in principle one could check here:
+	 *
+	 * - DHCPACK
+	 * - BOOT_ITEM
+	 * - RFC2132_SRV_ID
+	 * - UUID
+	 */
+	if (sport == PXE_BOOT_CLIENT) {
+		if (!find_vci_pxeclient(bootpreply->bp_vend + sizeof rfc1533_cookie)) {
+			return 0;
+		}
+		/* we only solicited this packet for the options and the bp_file */
+		memcpy((char*)&bootp_data, (char*)bootpreply, sizeof(struct bootpd_t));
+		memcpy(KERNEL_BUF, bootpreply->bp_file, sizeof(KERNEL_BUF));
+		return 1;
+	}			
+#endif /* PXE_DHCP_STRICT */
 	if (memcmp(&bootpreply->bp_siaddr, &zeroIP, sizeof(in_addr)) == 0)
 		return 0;
 	if ((memcmp(broadcast, bootpreply->bp_hwaddr, ETH_ALEN) != 0) &&
@@ -900,11 +1023,11 @@ static int await_bootp(int ival __unused, void *ptr __unused,
 		arptable[ARP_CLIENT].ipaddr.s_addr = bootpreply->bp_yiaddr.s_addr;
 #ifndef	NO_DHCP_SUPPORT
 		dhcp_addr.s_addr = bootpreply->bp_yiaddr.s_addr;
-#endif	/* NO_DHCP_SUPPORT */
+#endif	/* !NO_DHCP_SUPPORT */
 		netmask = default_netmask();
 		/* bootpreply->bp_file will be copied to KERNEL_BUF in the memcpy */
-		memcpy((char *)BOOTP_DATA_ADDR, (char *)bootpreply, sizeof(struct bootpd_t));
-		decode_rfc1533(BOOTP_DATA_ADDR->bootp_reply.bp_vend, 0,
+		memcpy((char *)&bootp_data, (char *)bootpreply, sizeof(struct bootpd_t));
+		decode_rfc1533(bootp_data.bootp_reply.bp_vend, 0,
 #ifdef	NO_DHCP_SUPPORT
 			       BOOTP_VENDOR_LEN + MAX_BOOTP_EXTLEN, 
 #else
@@ -932,7 +1055,7 @@ static int bootp(void)
 	int retry;
 #ifndef	NO_DHCP_SUPPORT
 	int reqretry;
-#endif	/* NO_DHCP_SUPPORT */
+#endif	/* !NO_DHCP_SUPPORT */
 	struct bootpip_t ip;
 	unsigned long  starttime;
 	unsigned char *bp_vend;
@@ -943,7 +1066,7 @@ static int bootp(void)
 	dhcp_machine_info[6] = ((nic.dev.devid.vendor_id) >> 8) & 0xff;
 	dhcp_machine_info[7] = nic.dev.devid.device_id & 0xff;
 	dhcp_machine_info[8] = ((nic.dev.devid.device_id) >> 8) & 0xff;
-#endif	/* NO_DHCP_SUPPORT */
+#endif	/* !NO_DHCP_SUPPORT */
 	memset(&ip, 0, sizeof(struct bootpip_t));
 	ip.bp.bp_op = BOOTP_REQUEST;
 	ip.bp.bp_htype = 1;
@@ -982,12 +1105,17 @@ static int bootp(void)
 			sizeof(struct bootpip_t), &ip);
 		remaining_time = rfc2131_sleep_interval(BOOTP_TIMEOUT, retry++);
 		stop_time = currticks() + remaining_time;
+#ifdef PXE_DHCP_STRICT
+		num_boot_menu = 0;
+#endif /* PXE_DHCP_STRICT */
 #ifdef	NO_DHCP_SUPPORT
 		if (await_reply(await_bootp, 0, NULL, remaining_time))
 			return(1);
 #else
 		while ( remaining_time > 0 ) {
 			if (await_reply(await_bootp, 0, NULL, remaining_time)){
+				if (arptable[ARP_CLIENT].ipaddr.s_addr)
+					break;
 			}
 			remaining_time = stop_time - currticks();
 		}
@@ -1026,6 +1154,10 @@ static int bootp(void)
 			dhcp_reply = 0;
 #ifdef PXE_EXPORT			
 			if ( arptable[ARP_PROXYDHCP].ipaddr.s_addr ) {
+#ifdef PXE_DHCP_STRICT
+				/* boot menu items must not be mixed from different servers */
+				num_boot_menu = 0;
+#endif /* PXE_DHCP_STRICT */
 				/* Construct the ProxyDHCPREQUEST packet */
 				memcpy(ip.bp.bp_vend, rfc1533_cookie, sizeof rfc1533_cookie);
 				memcpy(ip.bp.bp_vend + sizeof rfc1533_cookie, proxydhcprequest, sizeof proxydhcprequest);
@@ -1039,6 +1171,36 @@ static int bootp(void)
 					}
 				}
 			}
+#ifdef PXE_DHCP_STRICT
+			if (num_boot_menu) {
+				int i;
+				printf("\nreceived PXE boot menu:\n");
+				for (i = 0; i < num_boot_menu; ++i) {
+					printf("%d %s %@\n", pxe_boot_menu[i].id, pxe_boot_menu[i].text, pxe_boot_menu[i].ip.s_addr);
+				}
+				ip.bp.bp_siaddr.s_addr = zeroIP.s_addr;
+				ip.bp.bp_yiaddr.s_addr = zeroIP.s_addr;
+				ip.bp.bp_ciaddr.s_addr = arptable[ARP_CLIENT].ipaddr.s_addr;
+				memcpy(ip.bp.bp_vend, rfc1533_cookie, sizeof rfc1533_cookie);
+				bp_vend = ip.bp.bp_vend + sizeof rfc1533_cookie;
+				/* for now simply select the first entry */
+				*(uint16_t*)(pxedhcprequest + 7) = htons(pxe_boot_menu[0].id);
+				memcpy(bp_vend, pxedhcprequest, sizeof pxedhcprequest);
+				bp_vend += sizeof pxedhcprequest;
+				*bp_vend = RFC1533_END;
+				for (reqretry = 0; reqretry < MAX_BOOTP_RETRIES; ) {
+					printf("\nselecting boot item [%s] ... ", pxe_boot_menu[0].text);
+					arptable[ARP_SERVER].ipaddr.s_addr = pxe_boot_menu[0].ip.s_addr;
+					memset(&arptable[ARP_SERVER].node, 0, ETH_ALEN);
+					udp_transmit(pxe_boot_menu[0].ip.s_addr, PXE_BOOT_CLIENT,
+						     PXE_BOOT_SERVER, sizeof(struct bootpip_t), &ip);
+					timeout = rfc2131_sleep_interval(TIMEOUT, reqretry++);
+					if (await_reply(await_bootp, 0, NULL, timeout)) {
+						break;
+					}
+				}
+			}
+#endif /* PXE_DHCP_STRICT */
 #endif /* PXE_EXPORT */
 			return(1);
 		}
@@ -1047,6 +1209,9 @@ static int bootp(void)
 	}
 	return(0);
 }
+
+#endif /* USE_STATIC_BOOT_INFO */
+
 #endif	/* RARP_NOT_BOOTP */
 
 static uint16_t tcpudpchksum(struct iphdr *ip)
@@ -1609,6 +1774,64 @@ static int find_vci_etherboot(unsigned char *p)
 }
 #endif	/* REQUIRE_VCI_ETHERBOOT */
 
+#ifdef PXE_DHCP_STRICT
+/**************************************************************************
+DECODE_PXE_BOOT_SERVERS - Decodes the IP addresses of boot servers
+**************************************************************************/
+static void decode_pxe_boot_servers(unsigned char*p)
+{
+	unsigned short item = ntohs(*(unsigned short*)p);
+	unsigned char i;
+
+	for (i = 0; i < num_boot_menu; ++i) {
+		if (pxe_boot_menu[i].id == item)
+			break;
+	}
+	if (i == MAX_BOOT_MENU) return;
+	if (i == num_boot_menu) {
+		++num_boot_menu;
+		pxe_boot_menu[i].id = item;
+	}
+	memcpy(&pxe_boot_menu[i].ip, p + 3, sizeof(in_addr));
+}
+
+/**************************************************************************
+DECODE_PXE_BOOT_MENU - Decodes the boot menu items
+**************************************************************************/
+static void decode_pxe_boot_menu(unsigned char*p)
+{
+	unsigned short item = ntohs(*(unsigned short*)p);
+	unsigned char i, size;
+
+	for (i = 0; i < num_boot_menu; ++i) {
+		if (pxe_boot_menu[i].id == item)
+			break;
+	}
+	if (i == MAX_BOOT_MENU) return;
+	if (i == num_boot_menu) {
+		++num_boot_menu;
+		pxe_boot_menu[i].id = item;
+	}
+	memset(pxe_boot_menu[i].text, 0, sizeof(pxe_boot_menu[i].text));
+	size = p[2];
+	if (size > MAX_BOOT_ENTRY_LENGTH)
+		size = MAX_BOOT_ENTRY_LENGTH;
+	memcpy(pxe_boot_menu[i].text, p + 3, size);
+}
+#endif /* PXE_DHCP_STRICT */
+
+#if 0
+static void dump(unsigned char*p, unsigned char len)
+{
+	unsigned char*end = p+len;
+	printf("\n");
+	while (p<end) {
+		printf("%hhX ",*p);
+		++p;
+	}
+}
+#endif
+
 /**************************************************************************
 DECODE_RFC1533 - Decodes RFC1533 header
 **************************************************************************/
@@ -1618,6 +1841,10 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 	unsigned char	     *extpath = NULL;
 	unsigned char	     *endp;
 	static unsigned char in_encapsulated_options = 0;
+#ifdef PXE_DHCP_STRICT
+	static unsigned char in_pxe_encapsulated_options = 0;
+	unsigned char        pxeclient;
+#endif
 
 	if (eof == -1) {
 		/* Encapsulated option block */
@@ -1646,6 +1873,8 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 #else
 		vendorext_isvalid = 0;
 #endif
+		addparam = NULL;
+		addparamlen = 0;
 		if (memcmp(p, rfc1533_cookie, 4))
 			return(0); /* no RFC 1533 header found */
 		p += 4;
@@ -1656,7 +1885,7 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 				return(0); /* no RFC 1533 header found */
 			p += 4;
 			len -= 4; }
-		if (extend + len <= (unsigned char *)&(BOOTP_DATA_ADDR->bootp_extension[MAX_BOOTP_EXTLEN])) {
+		if (extend + len <= (unsigned char *)&(bootp_data.bootp_extension[MAX_BOOTP_EXTLEN])) {
 			memcpy(extend, p, len);
 			extend += len;
 		} else {
@@ -1668,6 +1897,9 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 	}
 	if (!eof)
 		return 1;
+#ifdef PXE_DHCP_STRICT
+	pxeclient = find_vci_pxeclient(p);
+#endif
 	while (p < endp) {
 		unsigned char c = *p;
 		if (c == RFC1533_PAD) {
@@ -1702,17 +1934,51 @@ int decode_rfc1533(unsigned char *p, unsigned int block, unsigned int len, int e
 			dhcp_reply=*(p+2);
 		else if (NON_ENCAP_OPT c == RFC2132_SRV_ID)
 			memcpy(&dhcp_server, p+2, sizeof(in_addr));
-#endif	/* NO_DHCP_SUPPORT */
+#endif	/* !NO_DHCP_SUPPORT */
 		else if (NON_ENCAP_OPT c == RFC1533_HOSTNAME) {
 			hostname = p + 2;
 			hostnamelen = *(p + 1);
 		}
+#ifdef PXE_DHCP_STRICT
+		else if (PXE_ENCAP_OPT c == PXE_BOOT_SERVERS) {
+			unsigned char *q = p + 2;
+			unsigned char *end = q + TAG_LEN(p);
+			while (q < end) {
+				decode_pxe_boot_servers(q);
+				q += q[2] * 4 + 3;
+			}
+		}
+		else if (PXE_ENCAP_OPT c == PXE_BOOT_MENU) {
+			unsigned char *q = p + 2;
+			unsigned char *end = q + TAG_LEN(p);
+			while (q < end) {
+				decode_pxe_boot_menu(q);
+				q += q[2] + 3;
+			}
+		}
+		else if (NON_ENCAP_OPT c == RFC1533_VENDOR && pxeclient) {
+			in_pxe_encapsulated_options = 1;
+			decode_rfc1533(p+2, 0, TAG_LEN(p), -1);
+			in_pxe_encapsulated_options = 0;
+		}
+#endif /* PXE_DHCP_STRICT */
 		else if (ENCAP_OPT c == RFC1533_VENDOR_MAGIC
 			 && TAG_LEN(p) >= 6 &&
 			  !memcmp(p+2,vendorext_magic,4) &&
 			  p[6] == RFC1533_VENDOR_MAJOR
 			)
 			vendorext_isvalid++;
+		else if (c == RFC1533_VENDOR_ADDPARM) {
+			/* This tag intentionally works for BOTH the encapsulated and
+			 * non-encapsulated case, since the current menu code (in mknbi)
+			 * creates this tag without encapsulation.  In the future both the
+			 * menu from mknbi and this code should learn about the proper
+			 * encapsulation (which will require substantial changes to various
+			 * stuff from mknbi, which will break compatibility with older
+			 * versions of Etherboot).  */
+			addparam = p + 2;
+			addparamlen = *(p + 1);
+		}
 		else if (NON_ENCAP_OPT c == RFC1533_VENDOR_ETHERBOOT_ENCAP) {
 			in_encapsulated_options = 1;
 			decode_rfc1533(p+2, 0, TAG_LEN(p), -1);
