@@ -18,10 +18,13 @@
 
 */
 
-static const char *version = "rhine.c v1.0.1 2003-02-06\n";
+static const char *version = "rhine.c v1.0.2 2004-10-29\n";
 
 /* A few user-configurable values. */
 
+// max time out delay time
+#define W_MAX_TIMEOUT	0x0FFFU
+
 /* Size of the in-memory receive ring. */
 #define RX_BUF_LEN_IDX	3	/* 0==8K, 1==16K, 2==32K, 3==64K */
 #define RX_BUF_LEN (8192 << RX_BUF_LEN_IDX)
@@ -46,6 +49,7 @@ static const char *version = "rhine.c v1.0.1 2003-02-06\n";
 #include "etherboot.h"
 #include "nic.h"
 #include "pci.h"
+#include "timer.h"
 
 /* define all ioaddr */
 
@@ -352,6 +356,7 @@ static const char *version = "rhine.c v1.0.1 2003-02-06\n";
 #define CFGD_GPIOEN		0x80
 #define CFGD_DIAG		0x40
 #define CFGD_MAGIC		0x10
+#define CFGD_RANDOM		0x08
 #define CFGD_CFDX		0x04
 #define CFGD_CEREN		0x02
 #define CFGD_CETEN		0x01
@@ -417,6 +422,26 @@ static const char *version = "rhine.c v1.0.1 2003-02-06\n";
 #define RX_RING_SIZE		2
 #define PKT_BUF_SZ		1536	/* Size of each temporary Rx buffer. */
 
+#define PCI_REG_MODE3		0x53
+#define MODE3_MIION		0x04    /* in PCI_REG_MOD3 OF PCI space */
+
+enum rhine_revs {
+    VT86C100A       = 0x00,
+    VTunknown0      = 0x20,
+    VT6102          = 0x40,
+    VT8231          = 0x50, /* Integrated MAC */
+    VT8233          = 0x60, /* Integrated MAC */
+    VT8235          = 0x74, /* Integrated MAC */
+    VT8237          = 0x78, /* Integrated MAC */
+    VTunknown1      = 0x7C,
+    VT6105          = 0x80,
+    VT6105_B0       = 0x83,
+    VT6105L 	    = 0x8A,
+    VT6107          = 0x8C,
+    VTunknown2      = 0x8E,
+    VT6105M         = 0x90,
+};
+
 /* Transmit and receive descriptors definition */
 
 struct rhine_tx_desc
@@ -653,7 +678,7 @@ static struct rhine_private
 }
 rhine;
 
-static void rhine_probe1 (struct nic *nic, int ioaddr,
+static void rhine_probe1 (struct nic *nic, struct pci_device *pci, int ioaddr,
 				 int chip_id, int options);
 static int QueryAuto (int);
 static int ReadMII (int byMIIIndex, int);
@@ -662,10 +687,21 @@ static void MIIDelay (void);
 static void rhine_init_ring (struct nic *dev);
 static void rhine_disable (struct dev *dev);
 static void rhine_reset (struct nic *nic);
-static int rhine_poll (struct nic *nic);
+static int rhine_poll (struct nic *nic, int retreive);
 static void rhine_transmit (struct nic *nic, const char *d, unsigned int t,
 			    unsigned int s, const char *p);
+static void reload_eeprom(int ioaddr);
 
+
+static void reload_eeprom(int ioaddr)
+{
+	int i;
+	outb(0x20, byEECSR);
+	/* Typically 2 cycles to reload. */
+	for (i = 0; i < 150; i++)
+		if (! (inb(byEECSR) & 0x20))
+			break;
+}
 /* Initialize the Rx and Tx rings, along with various 'dev' bits. */
 static void
 rhine_init_ring (struct nic *nic)
@@ -756,7 +792,8 @@ ReadMII (int byMIIIndex, int ioaddr)
     byMIItemp = inb (byMIICR);
     byMIItemp = byMIItemp & 0x40;
 
-    while (byMIItemp != 0)
+    load_timer2(2*TICKS_PER_MS);
+    while (byMIItemp != 0 && timer2_running())
     {
 	byMIItemp = inb (byMIICR);
 	byMIItemp = byMIItemp & 0x40;
@@ -796,7 +833,8 @@ WriteMII (char byMIISetByte, char byMIISetBit, char byMIIOP, int ioaddr)
     byMIItemp = inb (byMIICR);
     byMIItemp = byMIItemp & 0x40;
 
-    while (byMIItemp != 0)
+    load_timer2(2*TICKS_PER_MS);
+    while (byMIItemp != 0 && timer2_running())
     {
 	byMIItemp = inb (byMIICR);
 	byMIItemp = byMIItemp & 0x40;
@@ -825,7 +863,8 @@ WriteMII (char byMIISetByte, char byMIISetBit, char byMIIOP, int ioaddr)
     byMIItemp = inb (byMIICR);
     byMIItemp = byMIItemp & 0x20;
 
-    while (byMIItemp != 0)
+    load_timer2(2*TICKS_PER_MS);
+    while (byMIItemp != 0 && timer2_running())
     {
 	byMIItemp = inb (byMIICR);
 	byMIItemp = byMIItemp & 0x20;
@@ -851,13 +890,74 @@ MIIDelay (void)
     }
 }
 
+/* Offsets to the device registers. */
+enum register_offsets {
+        StationAddr=0x00, RxConfig=0x06, TxConfig=0x07, ChipCmd=0x08,
+        IntrStatus=0x0C, IntrEnable=0x0E,
+        MulticastFilter0=0x10, MulticastFilter1=0x14,
+        RxRingPtr=0x18, TxRingPtr=0x1C, GFIFOTest=0x54,
+        MIIPhyAddr=0x6C, MIIStatus=0x6D, PCIBusConfig=0x6E,
+        MIICmd=0x70, MIIRegAddr=0x71, MIIData=0x72, MACRegEEcsr=0x74,
+        ConfigA=0x78, ConfigB=0x79, ConfigC=0x7A, ConfigD=0x7B,
+        RxMissed=0x7C, RxCRCErrs=0x7E, MiscCmd=0x81,
+        StickyHW=0x83, IntrStatus2=0x84, WOLcrClr=0xA4, WOLcgClr=0xA7,
+        PwrcsrClr=0xAC,
+};
+
+/* Bits in the interrupt status/mask registers. */
+enum intr_status_bits {
+        IntrRxDone=0x0001, IntrRxErr=0x0004, IntrRxEmpty=0x0020,
+        IntrTxDone=0x0002, IntrTxError=0x0008, IntrTxUnderrun=0x0210,
+        IntrPCIErr=0x0040,
+        IntrStatsMax=0x0080, IntrRxEarly=0x0100,
+        IntrRxOverflow=0x0400, IntrRxDropped=0x0800, IntrRxNoBuf=0x1000,
+        IntrTxAborted=0x2000, IntrLinkChange=0x4000,
+        IntrRxWakeUp=0x8000,
+        IntrNormalSummary=0x0003, IntrAbnormalSummary=0xC260,
+        IntrTxDescRace=0x080000,        /* mapped from IntrStatus2 */
+        IntrTxErrSummary=0x082218,
+};
+#define DEFAULT_INTR (IntrRxDone | IntrRxErr | IntrRxEmpty| IntrRxOverflow | \
+                   IntrRxDropped | IntrRxNoBuf) 
+
+/***************************************************************************
+ IRQ - PXE IRQ Handler
+***************************************************************************/
+void rhine_irq ( struct nic *nic, irq_action_t action ) {
+    struct rhine_private *tp = (struct rhine_private *) nic->priv_data;
+    /* Enable interrupts by setting the interrupt mask. */
+    unsigned int intr_status;
+
+    switch ( action ) {
+        case DISABLE :
+        case ENABLE :
+            intr_status = inw(nic->ioaddr + IntrStatus);
+            /* On Rhine-II, Bit 3 indicates Tx descriptor write-back race. */
+
+            /* added comment by guard */
+            /* For supporting VT6107, please use revision id to recognize different chips in driver */
+            // if (tp->chip_id == 0x3065)
+            if( tp->chip_revision < 0x80 && tp->chip_revision >=0x40 )
+                intr_status |= inb(nic->ioaddr + IntrStatus2) << 16;
+                intr_status = (intr_status & ~DEFAULT_INTR);
+                if ( action == ENABLE ) 
+                    intr_status = intr_status | DEFAULT_INTR;
+                    outw(intr_status, nic->ioaddr + IntrEnable);
+                break;
+        case FORCE :
+            outw(0x0010, nic->ioaddr + 0x84);
+           break;
+        }
+}
+
 static int
 rhine_probe (struct dev *dev, struct pci_device *pci)
 {
     struct nic *nic = (struct nic *)dev;
+    struct rhine_private *tp = (struct rhine_private *) nic->priv_data;
     if (!pci->ioaddr)
 	return 0;
-    rhine_probe1 (nic, pci->ioaddr, pci->dev_id, -1);
+    rhine_probe1 (nic, pci, pci->ioaddr, pci->dev_id, -1);
 
     adjust_pci_device(pci);
     rhine_reset (nic);
@@ -865,6 +965,10 @@ rhine_probe (struct dev *dev, struct pci_device *pci)
     dev->disable  = rhine_disable;
     nic->poll     = rhine_poll;
     nic->transmit = rhine_transmit;
+    nic->irqno	  = pci->irq;
+    nic->irq      = rhine_irq;
+    nic->ioaddr   = tp->ioaddr;
+
 
     return 1;
 }
@@ -883,20 +987,25 @@ static void set_rx_mode(struct nic *nic __unused) {
 }
 
 static void
-rhine_probe1 (struct nic *nic, int ioaddr, int chip_id, int options)
+rhine_probe1 (struct nic *nic, struct pci_device *pci, int ioaddr, int chip_id, int options)
 {
     struct rhine_private *tp;
     static int did_version = 0;	/* Already printed version info. */
-    int i;
+    int i, ww;
     unsigned int timeout;
     int FDXFlag;
     int byMIIvalue, LineSpeed, MIICRbak;
+    uint8_t revision_id;    
+    unsigned char mode3_reg;
 
     if (rhine_debug > 0 && did_version++ == 0)
 	printf (version);
 
+    // get revision id.
+    pci_read_config_byte(pci, PCI_REVISION, &revision_id);
+
     /* D-Link provided reset code (with comment additions) */
-    if((chip_id != 0x3043) && (chip_id != 0x6100)) {
+    if (revision_id >= 0x40) {
 	unsigned char byOrgValue;
 	
 	if(rhine_debug > 0)
@@ -917,6 +1026,47 @@ rhine_probe1 (struct nic *nic, int ioaddr, int chip_id, int options)
 	
     }
 
+    /* Reset the chip to erase previous misconfiguration. */
+    outw(CR_SFRST, byCR0);
+    // if vt3043 delay after reset
+    if (revision_id <0x40) {
+       udelay(10000);
+    }
+    // polling till software reset complete
+    // W_MAX_TIMEOUT is the timeout period
+    for(ww = 0; ww < W_MAX_TIMEOUT; ww++) {
+        if ((inw(byCR0) & CR_SFRST) == 0)
+		break;
+        }
+
+    // issue AUTOLoad in EECSR to reload eeprom
+    outb(0x20, byEECSR );
+
+    // if vt3065 delay after reset
+    if (revision_id >=0x40) {
+	// delay 8ms to let MAC stable
+	mdelay(8);
+        /*
+         * for 3065D, EEPROM reloaded will cause bit 0 in MAC_REG_CFGA
+         * turned on.  it makes MAC receive magic packet
+         * automatically. So, we turn it off. (D-Link)
+         */
+         outb(inb(byCFGA) & 0xFE, byCFGA);
+    }
+
+    /* turn on bit2 in PCI configuration register 0x53 , only for 3065*/
+    if (revision_id >= 0x40) {
+        pci_read_config_byte(pci, PCI_REG_MODE3, &mode3_reg);
+        pci_write_config_byte(pci, PCI_REG_MODE3, mode3_reg|MODE3_MIION);
+    }
+
+
+    /* back off algorithm ,disable the right-most 4-bit off CFGD*/
+    outb(inb(byCFGD) & (~(CFGD_RANDOM | CFGD_CFDX | CFGD_CEREN | CFGD_CETEN)), byCFGD);
+
+    /* reload eeprom */
+    reload_eeprom(ioaddr);
+
     /* Perhaps this should be read from the EEPROM? */
     for (i = 0; i < ETH_ALEN; i++)
 	nic->node_addr[i] = inb (byPAR0 + i);
@@ -977,8 +1127,9 @@ rhine_probe1 (struct nic *nic, int ioaddr, int chip_id, int options)
     }
 
 
-    /* set MII 10 FULL ON */
-    WriteMII (17, 1, 1, ioaddr);
+    /* set MII 10 FULL ON, only apply in vt3043 */
+    if(chip_id == 0x3043)
+        WriteMII (0x17, 1, 1, ioaddr);
 
     /* turn on MII link change */
     MIICRbak = inb (byMIICR);
@@ -995,6 +1146,7 @@ rhine_probe1 (struct nic *nic, int ioaddr, int chip_id, int options)
     tp->chip_id = chip_id;
     tp->ioaddr = ioaddr;
     tp->phys[0] = -1;
+    tp->chip_revision = revision_id;
 
     /* The lower four bits are the media type. */
     if (options > 0)
@@ -1039,10 +1191,10 @@ rhine_reset (struct nic *nic)
     int rx_bufs_tmp, rx_bufs_tmp1;
     int tx_bufs_tmp, tx_bufs_tmp1;
 
-    static char buf1[RX_RING_SIZE * PKT_BUF_SZ + 32];
+    static char buf1[TX_RING_SIZE * PKT_BUF_SZ + 32];
     static char buf2[RX_RING_SIZE * PKT_BUF_SZ + 32];
     static char desc1[TX_RING_SIZE * sizeof (struct rhine_tx_desc) + 32];
-    static char desc2[TX_RING_SIZE * sizeof (struct rhine_tx_desc) + 32];
+    static char desc2[RX_RING_SIZE * sizeof (struct rhine_rx_desc) + 32];
 
     /* printf ("rhine_reset\n"); */
     /* Soft reset the chip. */
@@ -1102,12 +1254,9 @@ rhine_reset (struct nic *nic)
     /* Setup Multicast */	
     set_rx_mode(nic);
 
-    /* close IMR */
-    outw (0x0000, byIMR0);
-
-    /* set TCR RCR threshold */
-    outb (0x06, byBCR0);
-    outb (0x00, byBCR1);
+    /* set TCR RCR threshold to store and forward*/
+    outb (0x3e, byBCR0);
+    outb (0x38, byBCR1);
     outb (0x2c, byRCR);
     outb (0x60, byTCR);
     /* Set Fulldupex */
@@ -1123,18 +1272,37 @@ rhine_reset (struct nic *nic)
     CRbak = CRbak & 0xFFFB;	/* not CR_STOP */
     outw ((CRbak | CR_STRT | CR_TXON | CR_RXON | CR_DPOLL), byCR0);
 
-    /*set IMR to work */
-    outw (IMRShadow, byIMR0);
+    /* disable all known interrupt */
+    outw (0, byIMR0);
 }
+/* Beware of PCI posted writes */
+#define IOSYNC  do { readb(nic->ioaddr + StationAddr); } while (0)
 
 static int
-rhine_poll (struct nic *nic)
+rhine_poll (struct nic *nic, int retreive)
 {
     struct rhine_private *tp = (struct rhine_private *) nic->priv_data;
     int rxstatus, good = 0;;
 
     if (tp->rx_ring[tp->cur_rx].rx_status.bits.own_bit == 0)
     {
+        unsigned int intr_status;
+        /* There is a packet ready */
+        if(!retreive)
+            return 1;
+
+        intr_status = inw(nic->ioaddr + IntrStatus);
+        /* On Rhine-II, Bit 3 indicates Tx descriptor write-back race. */
+#if 0
+	if (tp->chip_id == 0x3065)
+	  intr_status |= inb(nic->ioaddr + IntrStatus2) << 16;
+#endif
+        /* Acknowledge all of the current interrupt sources ASAP. */
+        if (intr_status & IntrTxDescRace)
+           outb(0x08, nic->ioaddr + IntrStatus2);
+        outw(intr_status & 0xffff, nic->ioaddr + IntrStatus);
+	IOSYNC;
+
 	rxstatus = tp->rx_ring[tp->cur_rx].rx_status.lw;
 	if ((rxstatus & 0x0300) != 0x0300)
 	{
@@ -1157,6 +1325,11 @@ rhine_poll (struct nic *nic)
 	tp->cur_rx++;
 	tp->cur_rx = tp->cur_rx % RX_RING_SIZE;
     }
+        /* Acknowledge all of the current interrupt sources ASAP. */
+        outw(DEFAULT_INTR & ~IntrRxDone, nic->ioaddr + IntrStatus);
+
+        IOSYNC;
+
     return good;
 }
 
@@ -1168,6 +1341,9 @@ rhine_transmit (struct nic *nic,
     int ioaddr = tp->ioaddr;
     int entry;
     unsigned char CR1bak;
+    unsigned char CR0bak;
+    unsigned int nstype;
+
 
     /*printf ("rhine_transmit\n"); */
     /* setup ethernet header */
@@ -1178,12 +1354,14 @@ rhine_transmit (struct nic *nic,
 
     memcpy (tp->tx_buffs[entry], d, ETH_ALEN);	/* dst */
     memcpy (tp->tx_buffs[entry] + ETH_ALEN, nic->node_addr, ETH_ALEN);	/* src */
-    *((char *) tp->tx_buffs[entry] + 12) = t >> 8;	/* type */
-    *((char *) tp->tx_buffs[entry] + 13) = t;
+    
+    nstype=htons(t);
+    memcpy(tp->tx_buffs[entry] + 2 * ETH_ALEN, (char*)&nstype, 2);
+
     memcpy (tp->tx_buffs[entry] + ETH_HLEN, p, s);
     s += ETH_HLEN;
     while (s < ETH_ZLEN)
-	*((char *) tp->tx_buffs[entry] + ETH_HLEN + (s++)) = 0;
+	*((char *) tp->tx_buffs[entry] + (s++)) = 0;
 
     tp->tx_ring[entry].tx_ctrl.bits.tx_buf_size = s;
 
@@ -1203,9 +1381,24 @@ rhine_transmit (struct nic *nic,
     /*printf("td4=[%X]",inl(dwCurrentTDSE3)); */
 
     outb (CR1bak, byCR1);
-    /* Wait until transmit is finished */
-    while (tp->tx_ring[entry].tx_status.bits.own_bit != 0)
-	;
+    do
+    {
+        load_timer2(10*TICKS_PER_MS);
+        /* Wait until transmit is finished or timeout*/
+        while((tp->tx_ring[entry].tx_status.bits.own_bit !=0) && timer2_running())
+        ;
+
+        if(tp->tx_ring[entry].tx_status.bits.terr == 0)
+            break;
+
+        if(tp->tx_ring[entry].tx_status.bits.abt == 1)
+        {
+            // turn on TX
+            CR0bak = inb(byCR0);
+            CR0bak = CR0bak|CR_TXON;
+            outb(CR0bak,byCR0);
+        }
+    }while(0);
     tp->cur_tx++;
 
     /*outw(IMRShadow,byIMR0); */
